%=============================================================================
% data_object Data structure             
%=============================================================================  
% D=data_object(X, Y) 
%
% This is an object similar to the Spider data object (but simplified)
% http://www.kyb.mpg.de/bs/people/spider/
% 
% Data members
%    D.X : The original data or the predicted values (after passed through
%           a model, see linear_template)
%    D.Y : A label vector of binary target values 
%           (automatically turned to +-1 values by the constructor)
%           -- does not get changed by a model
%
% Parameter: 
%    D.standardized_data 
%           -- if 1, the data get standardized at construction time.
%           -- this is the default if the matrix is not a kernel matrix.
%    D.kernel_data
%           -- 1 if pd_check was run and the matrix was found semi-definite
%           positive (ran by the constructor).
%   Note: the order in which data are normalized and standardized may
%   matter.
%
% Methods:
%  ber, bac, auc, roc, pd_check, standardize, normalize, subdim
%
%==========================================================================
% Author of code: Isabelle Guyon -- isabelle@clopinet.com -- November 2010
%==========================================================================

classdef data_object
	properties (SetAccess = public)
        X=[];
        Y=[];
        standardized_data=0;
        normalized_data=0;
        kernel_data=[];
    end
    methods
        %%%%%%%%%%%%%%%%%%%
        %%% CONSTRUCTOR %%%
        %%%%%%%%%%%%%%%%%%%
        function D = data_object(X, Y) 
            if nargin<2 || isempty(Y), 
                Y=[]; 
            else
                % Verify sizes 
                if size(Y, 1)~=size(X, 1), fprintf('data_object warning: Inconsistent data and label sample number\n'); end
                %if size(Y, 2)~=1, fprintf('data_object warning: Too many columns in labels, single set of targets expected\n'); end
                ulabel=unique(Y);
                if length(ulabel)>2, 
                    fprintf('data_object warning: Too many label categories, binary targets expected\n');
                else % turn 0/1 targets to +-1 targets
                    Y(Y==0)=-1;
                end
            end
            D.X=X; 
            D.Y=Y;
            D.kernel_data=pd_check(D);
            if D.kernel_data, 
                D.standardized_data=0; 
                D.normalized_data=0;
            end
        end 
        
        %% Other methods
        function [area, sigma] =auc(D)
            %[area, sigma] =auc(D)
            % Compute the Area under the ROC curve
            area=0;
            if ~warning_not_result(D)
                [area, sigma] =auc(D.X, D.Y);
            end
        end
        
        function [acc, sigma]=bac(D)
            %[acc, sigma]=bac(D)
            % Compute the balanced accuracy
            acc=0;
            if ~warning_not_result(D)
                [acc, sigma]=bac(D.X, D.Y);
            end
        end
        
        function [errate, errate_pos, errate_neg, sigma]=ber(D)
            %[errate, errate_pos, errate_neg, sigma]=ber(D)
            % Compute the balanced error rate
            errate=0;
            if ~warning_not_result(D)
                [errate, errate_pos, errate_neg, sigma]=ber(D.X, D.Y);
            end
        end
        
        function h=roc(D, h, col)
            %h=roc(D, h, col)
            % Plot ROC curve
            % h=figure handle
            % col=color
            h=0;
            if ~warning_not_result(D)
                if nargin<2, h=figure; end
                if nargin<3, col='k'; end          
                h=roc(D.X, D.Y, [],[],[],[],h,col);
            end
        end
        
        function w=warning_not_result(D)
            w=0;
            if any(size(D.X)~=size(D.Y)),
                w=1;
                fprintf('data_object warning: Size of X and Y must be the same to test a result object\n'); 
            end
        end
        
        function t=pd_check(D)
            %t=pd_check(D)
            % Check whether the matrix is positive semi-definite
            t=D.kernel_data;
            if isempty(t)
                t=pd_check(D.X);
            else
                if t && (size(D, 1)~=size(D, 2))
                    error('pd_check: wrong subdim');
                end
            end
        end        
        
        function D=kernelize(D)
            %D=kernelize(D)
            % Replace D.X by D.X*D.X'
            D.X=D.X*D.X';
            D.kernel_data=1;
        end
        
        function D=standardize(D)
            %D=standardize(D)
            % For each feature, subtract mean and divide by stdev
            D.X=standardize_data(D.X);
            D.standardized_data=1;
        end
        
        function D=normalize(D, norm)
            %D=normalize(D, norm)
            % For each pattern, divide by the sum of the absolute values
            % or by the Euclidean norm.
            if nargin<2, norm=1; end
            D.X=normalize_data(D.X, norm);
            D.normalized_data=1;
        end
        
        function D=subdim(D, pidx, fidx, lidx)
            %D=subdim(D, pidx, fdix, ldix)
            % Sub-dimension data to a matrix (pidx x fidx).
            % If either is set to [], the dimension is not changed
            if nargin<2, pidx=[]; end
            if nargin<3, fidx=[]; end
            if nargin<4, lidx=[]; end
            if isempty(pidx), pidx=1:size(D.X, 1); end
            if isempty(fidx), fidx=1:size(D.X, 2); end
            if isempty(lidx), lidx=1:size(D.Y, 2); end
            D.X=D.X(pidx, fidx);
            D.Y=D.Y(pidx, lidx);
        end   

        function t=am_i_data(D)
		% Spider compatibility function
		t=[];
	  end
            
        function t=isdeferred(D)
		% Spider compatibility function
		t=[];
	  end

        function t=waitcollect(D)
		% Spider compatibility function
		t=[];
	  end

        function X=get_x(D)
		% Spider compatibility function
		X=D.X;
	  end

        function Y=get_y(D)
		% Spider compatibility function
		Y=D.Y;
	  end

        function D=set_x(D, X)
		% Spider compatibility function
		D.X=X;
	  end

        function D=set_y(D, Y)
		% Spider compatibility function
		D.Y=Y;
	  end

        function n=get_name(D)
		% Spider compatibility function
		n=class(D);
	  end

        function D=set_name(D, n)
		% Spider compatibility function
	  end
    end %methods
end % classdef
