function [x, y, e] =make_learning_curve(a, X, Y, min_repeat, max_repeat, ebar, max_point_num)
%[x, y, e] =make_learning_curve(a, X, Y, min_repeat, max_repeat, ebar, max_point_num)
% Make the learning curve 
% Inputs:
% a -- an machine learning algorithm (CLOP object)
% X -- data matrix
% Y -- labels
% Returns:
% x -- Number of samples
% y -- Performance (AUC)
% e -- error bar

% Author: Isabelle Guyon -- November 2010 -- isabelle@clopinet.com

if nargin<7, max_point_num=[]; end

debug=0;
X=double(X);

% Verify dimensions and set target values
[p, n]=size(X);
[pp, cn]=size(Y);
sep_num=cn;
if pp~=p, error('Size mismatch'); end
if cn==2 && all(sum(Y,2)) % If only 2 classes, the 2nd col is the same as the first but opposite
    Y=Y(:,1);
    sep_num=1;
end
Y(Y==0)=-1;

% Identify the algorithm
algo=class(a);

% Create the data matrices (Y at this stage is still multi-column)
D=data_struct(X, Y);
feat_num=size(D.X, 2);
K=[];
if ~pd_check(D)
    K=kernelize(D);
end

% Load random splits (these are the same for everyone)
RP=[];
if exist('RP.mat', 'file')
    load('RP');
    [rp, mr]=size(RP);
    if rp<p
        fprintf('make_learning_curve::warning: RP too small\n');
        RP=[];
    else
        max_repeat=min(max_repeat, mr);
        RP=ceil(RP/(rp/p));
        if debug, 
            fprintf('make_learning_curve: using RP of dim %dx%d min=%d max=%d, max_repeat=%d\n', rp, mr, min(min(RP)), max(max(RP)), max_repeat);
        end
    end
else
    fprintf('make_learning_curve::warning: no RP file found\n');
end

% Sample sizes scaled in log2
m=floor(log2(p));
x=2.^[0:m];
if x(end)~=p
    x=[x p];
end
x=x(1:end-1); % need to remove the last point
if isempty(max_point_num)
    point_num=length(x);
else
    point_num=min(length(x), max_point_num);
end

% Loop over the sample sizes
x=x(1:point_num);
y=zeros(size(x));
e=zeros(size(x));
for k=1:point_num
    if debug, fprintf('-------------------- Point %d ----------------------\n', k) ; end
    A=zeros(sep_num,1);
    E=zeros(sep_num,1);
    e(k)=Inf; 
    % Loop over number of "1 vs all" separations
    for j=1:sep_num
        if debug, fprintf(' sep %d -- ', j); end
        repnum=0;
        area=[];
        % Loop over repeats (floating number of repeats)
        while repnum<min_repeat || (E(j)>ebar && repnum<max_repeat)
            repnum=repnum+1;
            if debug, fprintf('repeat %d **\n', repnum); end
            if isempty(RP)
                rp=randperm(p);
            else
                rp=RP(1:p, repnum);
            end
            tr_idx=rp(1:x(k));
            te_idx=rp(x(k)+1:end);
            if pd_check(D) % kernelized version
                Dtr=subdim(D, tr_idx, tr_idx, j);
                Dte=subdim(D, te_idx, tr_idx, j);
            elseif x(k)< feat_num || strcmp(algo, 'lssvm') % kernelized too (for speed reason)
                Dtr=subdim(K, tr_idx, tr_idx, j);
                Dte=subdim(K, te_idx, tr_idx, j);               
            else % primal version 
                Dtr=subdim(D, tr_idx, [], j);
                Dte=subdim(D, te_idx, [], j);
            end
            [d, m]=train(a, Dtr);
            d1=test(m, Dte);
            area(repnum)=auc(d1);
            E(j)=std(area)/sqrt(repnum);         
        end % repnum loop
        A(j)=mean(area); 
    end % for j=1:sep_num
    e(k)=mean(E);
    y(k)=mean(A);   
    if debug, 
        fprintf('==> %d repeats, auc=%5.2f+-%5.2f -----------------\n', repnum, y(k), e(k));  
    end
end % Loop over k

% Add point with 0 examples
x=[0 x];
P=0.5;
y=[P y];
e=[sqrt(P*(1-P)/p) e];


