function [center, Y] = uns_kmeans(X, cnum, itmax, center0)
%[center, Y] = uns_kmeans(X, cnum, itmax, center0)
% K means clustering / unsupersised classifier
% Inputs:
%		X -- matrix of line training vectors
%		cnum -- number of clusters
% Returns:
% 		center -- matrix of line cluster centers
%		Y -- class labels of X
% Optional:
%       itmax -- maximum number of iterations
%       center0 -- initial centers

% Isabelle Guyon -- December 1999 -- isabelle@clopinet.com

if (nargin>4 || nargin<2), help uns_kmeans; return; end
if nargin<2, cnum=20; end
if nargin<3, itmax=5; end
if nargin<4, center0=[]; end

debug=0;

% Initialization
fprintf('\nInitializing');
n = size(X, 1);
if cnum >= n, center = X; Y = 1:n; return; end
center = zeros(cnum,size(X,2));
if isempty(center0),
   % Start with random class assignments
	% Split data into cnum subsets of identical size
	R = randperm(n);
	% Number of patterns per subset
	p = floor(n/cnum);
	% Subset limiters:
	lb(1)=1; ub(cnum)=n;
	for k=1:cnum-1
   	ub(k)=k*p;
   	lb(k+1)=ub(k)+1;
   end
	% Initial class assignments
	Y=zeros(size(X,1),1);
	for k=1:cnum
   	Y(R(lb(k):ub(k))) = k;
   end
else
   % Start with initial cluster centers
   center=center0;
   % Compute initial class assignment
   % Compute similarity measures (matrix n x cnum)
   DP = sv_dp('euclid2',X,center);
   % Assign patterns to their closest cluster center
	[score, Y] = min(DP,[],2);
end
% (Re)compute initial cluster centers
for k=1:cnum
   k_idx=find(Y==k);
   center(k,:) = mean(X(k_idx(1:min(length(k_idx), 100)),:), 1); 
end

if debug
    h=figure; hold on;
    for k=1:size(center,1)
        subplot(5, cnum/5, k);
        show_patt(center(k,:), [], h);
        title('Initialization');
    end
end

old_Y = zeros(size(Y));
% Loop until stopping criterion is met
it=0;
while (any(Y ~= old_Y)) && it<itmax
   fprintf('\nIteration %d/%d: \n', it+1, itmax);
   old_Y=Y;
   fprintf('     Compute similarity measures (matrix %d x %d)\n', n, cnum);
   DP = sv_dp('euclid2',X,center);
   fprintf('     Assign patterns to their closest cluster center\n');
   [score, Y] = min(DP,[],2);
   fprintf('     Recompte the cluster centers\n');
   empty=ones(cnum,1);
   for k=1:cnum
      k_idx=find(Y==k);
      if ~isempty(k_idx)
          center(k,:) = mean(X(k_idx,:), 1); 
          empty(k)=0;
      end
   end
   good_idx=find(empty==0);
   center=center(good_idx,:);
   cnum=length(good_idx);
   it=it+1;
   fprintf('... completed.\n');
   
   if debug
       h=figure; hold on;
       for k=1:size(center,1)
           subplot(5, cnum/5, k);
           show_patt(center(k,:), [], h);
           title(['Iteration', num2str(it)]);
       end
   end
end

